@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 # This file is part of beets.
 # Copyright 2015, Adrian Sampson.
 #
@@ -41,6 +42,10 @@ IMAGE_EXTENSIONS = ['png', 'jpg', 'jpeg']
 CONTENT_TYPES = ('image/jpeg', 'image/png')
 DOWNLOAD_EXTENSION = '.jpg'
 
+CANDIDATE_BAD = 0
+CANDIDATE_EXACT = 1
+CANDIDATE_DOWNSCALE = 2
+
 
 def _logged_get(log, *args, **kwargs):
     """Like `requests.get`, but logs the effective URL to the specified
@@ -152,34 +157,6 @@ class AlbumArtOrg(ArtSource):
             self._log.debug(u'no image found on page')
 
 
-class GoogleImages(ArtSource):
-    URL = 'https://ajax.googleapis.com/ajax/services/search/images'
-
-    def get(self, album):
-        """Return art URL from google.org given an album title and
-        interpreter.
-        """
-        if not (album.albumartist and album.album):
-            return
-        search_string = (album.albumartist + ',' + album.album).encode('utf-8')
-        response = self.request(self.URL, params={
-            'v': '1.0',
-            'q': search_string,
-            'start': '0',
-        })
-
-        # Get results using JSON.
-        try:
-            results = response.json()
-            data = results['responseData']
-            dataInfo = data['results']
-            for myUrl in dataInfo:
-                yield myUrl['unescapedUrl']
-        except:
-            self._log.debug(u'error scraping art page')
-            return
-
-
 class ITunesStore(ArtSource):
     # Art from the iTunes Store.
     def get(self, album):
@@ -191,11 +168,19 @@ class ITunesStore(ArtSource):
         try:
             # Isolate bugs in the iTunes library while searching.
             try:
-                itunes_album = itunes.search_album(search_string)[0]
+                results = itunes.search_album(search_string)
             except Exception as exc:
                 self._log.debug('iTunes search failed: {0}', exc)
                 return
 
+            # Get the first match.
+            if results:
+                itunes_album = results[0]
+            else:
+                self._log.debug('iTunes search for {:r} got no results',
+                                search_string)
+                return
+
             if itunes_album.get_artwork()['100']:
                 small_url = itunes_album.get_artwork()['100']
                 big_url = small_url.replace('100x100', '1200x1200')
@@ -375,7 +360,7 @@ class FileSystem(ArtSource):
 
 # Try each source in turn.
 
-SOURCES_ALL = [u'coverart', u'itunes', u'amazon', u'albumart', u'google',
+SOURCES_ALL = [u'coverart', u'itunes', u'amazon', u'albumart',
                u'wikipedia']
 
 ART_SOURCES = {
@@ -383,7 +368,6 @@ ART_SOURCES = {
     u'itunes': ITunesStore,
     u'albumart': AlbumArtOrg,
     u'amazon': Amazon,
-    u'google': GoogleImages,
     u'wikipedia': Wikipedia,
 }
 
@@ -430,6 +414,9 @@ class FetchArtPlugin(plugins.BeetsPlugin, RequestMixin):
     def fetch_art(self, session, task):
         """Find art for the album being imported."""
         if task.is_album:  # Only fetch art for full albums.
+            if task.album.artpath and os.path.isfile(task.album.artpath):
+                # Album already has art (probably a re-import); skip it.
+                return
             if task.choice_flag == importer.action.ASIS:
                 # For as-is imports, don't search Web sources for art.
                 local = True
@@ -505,24 +492,49 @@ class FetchArtPlugin(plugins.BeetsPlugin, RequestMixin):
             return None
 
     def _is_valid_image_candidate(self, candidate):
-        if not candidate:
-            return False
+        """Determine whether the given candidate artwork is valid based on
+        its dimensions (width and ratio).
 
-        if not (self.enforce_ratio or self.minwidth):
-            return True
+        Return `CANDIDATE_BAD` if the file is unusable.
+        Return `CANDIDATE_EXACT` if the file is usable as-is.
+        Return `CANDIDATE_DOWNSCALE` if the file must be resized.
+        """
+        if not candidate:
+            return CANDIDATE_BAD
+
+        if not (self.enforce_ratio or self.minwidth or self.maxwidth):
+            return CANDIDATE_EXACT
 
         # get_size returns None if no local imaging backend is available
         size = ArtResizer.shared.get_size(candidate)
+        self._log.debug('image size: {}', size)
 
         if not size:
-            self._log.warning(u'could not verify size of image: please see '
-                              u'documentation for dependencies. '
+            self._log.warning(u'Could not get size of image (please see '
+                              u'documentation for dependencies). '
                               u'The configuration options `minwidth` and '
                               u'`enforce_ratio` may be violated.')
-            return True
+            return CANDIDATE_EXACT
 
-        return size and size[0] >= self.minwidth and \
-            (not self.enforce_ratio or size[0] == size[1])
+        # Check minimum size.
+        if self.minwidth and size[0] < self.minwidth:
+            self._log.debug('image too small ({} < {})',
+                            size[0], self.minwidth)
+            return CANDIDATE_BAD
+
+        # Check aspect ratio.
+        if self.enforce_ratio and size[0] != size[1]:
+            self._log.debug('image is not square ({} != {})',
+                            size[0], size[1])
+            return CANDIDATE_BAD
+
+        # Check maximum size.
+        if self.maxwidth and size[0] > self.maxwidth:
+            self._log.debug('image needs resizing ({} > {})',
+                            size[0], self.maxwidth)
+            return CANDIDATE_DOWNSCALE
+
+        return CANDIDATE_EXACT
 
     def art_for_album(self, album, paths, local_only=False):
         """Given an Album object, returns a path to downloaded art for the
@@ -532,6 +544,7 @@ class FetchArtPlugin(plugins.BeetsPlugin, RequestMixin):
         are made.
         """
         out = None
+        check = None
 
         # Local art.
         cover_names = self.config['cover_names'].as_str_seq()
@@ -540,7 +553,8 @@ class FetchArtPlugin(plugins.BeetsPlugin, RequestMixin):
         if paths:
             for path in paths:
                 candidate = self.fs_source.get(path, cover_names, cautious)
-                if self._is_valid_image_candidate(candidate):
+                check = self._is_valid_image_candidate(candidate)
+                if check:
                     out = candidate
                     self._log.debug('found local image {}', out)
                     break
@@ -552,12 +566,13 @@ class FetchArtPlugin(plugins.BeetsPlugin, RequestMixin):
                 if self.maxwidth:
                     url = ArtResizer.shared.proxy_url(self.maxwidth, url)
                 candidate = self._fetch_image(url)
-                if self._is_valid_image_candidate(candidate):
+                check = self._is_valid_image_candidate(candidate)
+                if check:
                     out = candidate
                     self._log.debug('using remote image {}', out)
                     break
 
-        if self.maxwidth and out:
+        if self.maxwidth and out and check == CANDIDATE_DOWNSCALE:
             out = ArtResizer.shared.resize(self.maxwidth, out)
 
         return out
@@ -567,7 +582,7 @@ class FetchArtPlugin(plugins.BeetsPlugin, RequestMixin):
         fetchart CLI command.
         """
         for album in albums:
-            if album.artpath and not force:
+            if album.artpath and not force and os.path.isfile(album.artpath):
                 message = ui.colorize('text_highlight_minor', 'has album art')
             else:
                 # In ordinary invocations, look for images on the
